import { __awaiter } from 'tslib';
import { jsx, jsxs, Fragment } from 'react/jsx-runtime';
import axios from 'axios';
import { useState, useEffect } from 'react';
import { defineComponent, useEvaluate } from '@qorebase/app-component';
import { useDisclosure, Center, Box, SlideFade, FormLabel, Popover, PopoverTrigger, Stack, InputGroup, InputRightElement, Input, PopoverContent, PopoverArrow, PopoverBody, CircularProgress, VStack, StackDivider, Flex, Text, Spinner } from '@chakra-ui/react';

// import { FiSearch } from 'react-icons/fi';
// import { BiMapPin } from 'react-icons/bi';
// import { FaCity } from 'react-icons/fa';
const LoaderItem = () => (jsx(Center, { children: jsx(Spinner, { color: 'var(--chakra-colors-qore-primary)' }) }));
const LoaderBox = () => (jsx(Center, { children: jsx(Spinner, { color: 'var(--chakra-colors-qore-primary)' }) }));
var customSelectComponent = defineComponent({
    name: "Custom Select Component V.1.0.1",
    type: "none",
    icon: "none",
    group: "text",
    defaultProps: {
        server: "Server URL",
        secret: "Admin Secret",
        table: "Table Name",
        grid: "Grid Name",
        params: "Parameter Name",
        items: "Items Variable"
    },
    propDefinition: {
        server: { group: "Text", type: "expression", options: {} },
        secret: { group: "Text", type: "expression", options: {} },
        table: { group: "Text", type: "expression", options: {} },
        grid: { group: "Text", type: "expression", options: {} },
        params: { group: "Text", type: "expression", options: {} },
        items: { group: "Text", type: "expression", options: {} },
    },
    Component: (props) => {
        const server = useEvaluate(props.properties.server);
        const secret = useEvaluate(props.properties.secret);
        const table = useEvaluate(props.properties.table);
        const grid = useEvaluate(props.properties.grid);
        const params = useEvaluate(props.properties.params);
        const items = useEvaluate(props.properties.items);
        const headers = {
            accept: '*/*',
            'x-qore-engine-admin-secret': secret,
        };
        // Status
        const [loaderBox, setLoaderBox] = useState(false);
        const [loaderSearch, setSearchLoader] = useState(false);
        // Handle Animate
        const animateBox = useDisclosure();
        // State Data
        const [initialData, setInitialData] = useState([]);
        const [filteredData, setFilteredData] = useState([]);
        const [searchData, setSearchData] = useState("");
        const [selectedItems, setSelectedItems] = useState('');
        // Handle Dropdown
        const filterPopover = useDisclosure();
        const handleSelectedItem = (selectedItems) => __awaiter(void 0, void 0, void 0, function* () {
            if (selectedItems) {
                yield setSelectedItems(`${selectedItems[`${items}`]}`);
                yield filterPopover.onClose();
            }
        });
        useEffect(() => {
            getData();
        }, []);
        const getData = () => __awaiter(void 0, void 0, void 0, function* () {
            const response = yield axios.post(`${server}/v1/execute`, {
                "operations": [
                    {
                        "operation": "Select",
                        "instruction": {
                            "table": `${table}`,
                            "name": "selectData",
                        }
                    }
                ]
            }, { headers: headers });
            setInitialData(response.data.results.selectData);
            setLoaderBox(false);
            animateBox.onOpen();
        });
        useEffect(() => {
            const callSearchService = () => __awaiter(void 0, void 0, void 0, function* () {
                setSearchLoader(true);
                const getData = yield axios.get(`${server}/v1/grid/${grid}?limit=500&params[${params}]=${searchData}`, { headers: headers });
                yield setFilteredData(getData.data.items);
                setSearchLoader(false);
            });
            let debouncer = setTimeout(() => {
                callSearchService();
            }, 1000);
            return () => {
                clearTimeout(debouncer);
            };
        }, [searchData]);
        return (jsx(Center, Object.assign({ minH: '100vh', w: '100%' }, { children: loaderBox ? (jsx(Box, Object.assign({ w: '100%' }, { children: jsx(LoaderItem, {}) }))) : (jsx(Box, Object.assign({ position: 'relative', w: 'auto', pb: '40px' }, { children: jsx(SlideFade, Object.assign({ in: animateBox.isOpen, offsetY: '20px' }, { children: jsxs(Box, Object.assign({ boxShadow: 'lg', position: 'relative', p: '20px', backgroundColor: 'white' }, { children: [jsxs(Box, { children: [jsx(Box, Object.assign({ color: '#171717', px: '10px', my: '5px', fontSize: '16px', fontWeight: 700 }, { children: "Select Component" })), jsx(Center, Object.assign({ flexWrap: 'wrap', justifyContent: 'space-between' }, { children: jsx(Box, Object.assign({ w: '100%', my: '5px' }, { children: initialData && initialData.length > 0 ? (jsxs(Box, { children: [jsx(FormLabel, Object.assign({ ml: '10px', mb: '10px' }, { children: "City" })), jsxs(Popover, Object.assign({ isOpen: filterPopover.isOpen, onOpen: filterPopover.onOpen, onClose: () => {
                                                            filterPopover.onClose();
                                                            setSearchData('');
                                                        }, placement: 'bottom' }, { children: [jsx(PopoverTrigger, { children: jsx(Stack, Object.assign({ spacing: 3, px: '10px' }, { children: jsxs(InputGroup, { children: [jsx(InputRightElement, { pointerEvents: "none" }), jsx(Input, Object.assign({ color: 'gray.500', as: 'button', fontSize: '14px', textAlign: 'left', border: "1px", borderColor: "#E9E9E9", _focus: { outline: "none" } }, { children: selectedItems == '' ? "Select" : selectedItems }))] }) })) }), jsxs(PopoverContent, Object.assign({ w: '100%', minW: '454px', boxShadow: "base", maxH: "420px", overflowY: "hidden", _focus: { outline: "none" } }, { children: [jsx(PopoverArrow, {}), jsx(PopoverBody, Object.assign({ px: "0" }, { children: jsx(Center, Object.assign({ justifyContent: 'space-between' }, { children: jsxs(Box, Object.assign({ w: '100%' }, { children: [jsx(Stack, Object.assign({ spacing: 3, px: "10px" }, { children: jsxs(InputGroup, { children: [jsx(InputRightElement, { pointerEvents: "none", children: loaderSearch ?
                                                                                                        jsx(CircularProgress, { size: '20px', mx: 'auto', isIndeterminate: true, color: '#c6b378' })
                                                                                                        :
                                                                                                            // <FiSearch color="rgba(162, 162, 162, 1)" />
                                                                                                            jsx(Fragment, {}) }), jsx(Input, { _focus: { outline: "none" }, placeholder: "Search by City", fontSize: "12px", onChange: (e) => {
                                                                                                        setSearchData(e.target.value);
                                                                                                    } })] }) })), filteredData && filteredData.length > 0 && (jsx(VStack, Object.assign({ w: 'auto', divider: jsx(StackDivider, { borderColor: 'gray.200' }), spacing: 4, align: 'stretch', mt: 0, className: "filter-checkbox-wrapper", overflowY: "scroll", maxH: "300px" }, { children: jsx(Box, { children: filteredData.map((el, index) => {
                                                                                                var _a;
                                                                                                return (jsx(Box, Object.assign({ p: 3, onClick: () => handleSelectedItem(el), cursor: 'pointer', _hover: {
                                                                                                        backgroundColor: "rgba(249, 247, 242, 1)",
                                                                                                    } }, { children: jsx(Flex, Object.assign({ flexDir: "row", justifyContent: 'space-between' }, { children: jsx(Box, { children: jsx(Flex, Object.assign({ alignItems: 'center', flexDir: "row" }, { children: jsx(Text, Object.assign({ mx: '5px', fontSize: '12px', textTransform: 'capitalize' }, { children: (_a = el[`${items}`]) === null || _a === void 0 ? void 0 : _a.toLowerCase() })) })) }) })) }), index));
                                                                                            }) }) })))] })) })) }))] }))] }))] })) : (jsx(Box, { children: jsxs(Stack, Object.assign({ spacing: 3, px: '10px' }, { children: [jsx(FormLabel, Object.assign({ mb: 0 }, { children: "City" })), jsxs(InputGroup, { children: [jsx(InputRightElement, { pointerEvents: "none", children: 
                                                                    // <FaCity color="rgba(162, 162, 162, 1)" />
                                                                    jsx(Fragment, {}) }), jsx(Input, { _placeholder: { opacity: 1, color: 'gray.500' }, _focus: { outline: "none" }, placeholder: "Fill City", fontSize: "12px" })] })] })) })) })) }))] }), loaderBox && (jsx(Box, Object.assign({ left: '0', top: '0', position: 'absolute', w: '100%', h: '100%', backgroundColor: 'rgba(255,255,255,.7)' }, { children: jsx(LoaderBox, {}) })))] })) })) }))) })));
    },
});

export { customSelectComponent as default };
