import { createRequire } from "node:module";
import { nodeResolve } from "@rollup/plugin-node-resolve";
import typescript from "@rollup/plugin-typescript";
import commonjs from "@rollup/plugin-commonjs";
import bundleCommonjs from "@qorebase/rollup-plugin-bundle-commonjs";

const require = createRequire(import.meta.url);
const packageJson = require("./package.json");

const filename = packageJson.module.slice(
  "lib/".length,
  packageJson.module.indexOf(".")
);

/**
 * @type {import('rollup').RollupOptions}
 */
const rollupOptions = {
  input: `src/${filename}.tsx`,
  output: {
    file: `lib/${filename}.js`,
    format: "esm",
  },
  treeshake: {
    moduleSideEffects: false,
  },
  external: ["react", "react/jsx-runtime"],
  plugins: [
    commonjs(),
    bundleCommonjs(),
    nodeResolve({
      browser: true,
    }),
    typescript(),
  ],
};

export default rollupOptions;
