import axios from 'axios';
import { useEffect, useState } from "react";
import { defineComponent, useEvaluate } from "@qorebase/app-component";
import {
    Box,
    Stack,
    Text,
    Flex,
    Center,
    Spinner,
    InputGroup,
    Input,
    InputRightElement,
    CircularProgress,
    VStack,
    StackDivider,
    Popover,
    PopoverTrigger,
    PopoverContent,
    PopoverBody,
    PopoverArrow,
    useDisclosure,
    FormLabel,
    SlideFade,
} from '@chakra-ui/react';
// import { FiSearch } from 'react-icons/fi';
// import { BiMapPin } from 'react-icons/bi';
// import { FaCity } from 'react-icons/fa';

const LoaderItem: React.FC = () => (
    <Center>
        <Spinner color='var(--chakra-colors-qore-primary)' />
    </Center>
)

const LoaderBox: React.FC = () => (
    <Center>
        <Spinner color='var(--chakra-colors-qore-primary)' />
    </Center>
)


export default defineComponent({
    name: "Custom Select Component V.1.0.1",
    type: "none",
    icon: "none",
    group: "text",
    defaultProps: {
        server: "Server URL",
        secret: "Admin Secret",
        table: "Table Name",
        grid: "Grid Name",
        params: "Parameter Name",
        items: "Items Variable"
    },
    propDefinition: {
        server: { group: "Text", type: "expression", options: {} },
        secret: { group: "Text", type: "expression", options: {} },
        table: { group: "Text", type: "expression", options: {} },
        grid: { group: "Text", type: "expression", options: {} },
        params: { group: "Text", type: "expression", options: {} },
        items: { group: "Text", type: "expression", options: {} },
    },
    Component: (props) => {
        const server = useEvaluate(props.properties.server);
        const secret = useEvaluate(props.properties.secret);
        const table = useEvaluate(props.properties.table);
        const grid = useEvaluate(props.properties.grid);
        const params = useEvaluate(props.properties.params);
        const items = useEvaluate(props.properties.items);

        const headers = {
            accept: '*/*',
            'x-qore-engine-admin-secret': secret,
        }

        // Status
        const [loaderBox, setLoaderBox] = useState<boolean>(false);
        const [loaderSearch, setSearchLoader] = useState<boolean>(false);

        // Handle Animate
        const animateBox = useDisclosure();

        // State Data
        const [initialData, setInitialData] = useState<any[]>([]);
        const [filteredData, setFilteredData] = useState<any[]>([]);
        const [searchData, setSearchData] = useState<string>("");

        const [selectedItems, setSelectedItems] = useState<string>('');

        // Handle Dropdown
        const filterPopover = useDisclosure();
        const handleSelectedItem = async (selectedItems: any) => {
            if (selectedItems) {
                await setSelectedItems(`${selectedItems[`${items}`]}`);
                await filterPopover.onClose();
            }
        };

        useEffect(() => {
            getData();
        }, [])

        const getData = async () => {
            const response = await axios.post(`${server}/v1/execute`, {
                "operations": [
                    {
                        "operation": "Select",
                        "instruction": {
                            "table": `${table}`,
                            "name": "selectData",
                        }
                    }
                ]
            }, { headers: headers })

            setInitialData(response.data.results.selectData)
            setLoaderBox(false);
            animateBox.onOpen()
        }

        useEffect(() => {
            const callSearchService = async () => {
                setSearchLoader(true);
                const getData = await axios.get(`${server}/v1/grid/${grid}?limit=500&params[${params}]=${searchData}`, { headers: headers });   
                await setFilteredData(getData.data.items);
                setSearchLoader(false)
            }

            let debouncer = setTimeout(() => {
                callSearchService();
            }, 1000);

            return () => {
                clearTimeout(debouncer);
            }
            
        }, [searchData])

        return (
            <Center minH={'100vh'} w={'100%'}>
                {
                    loaderBox ? (
                        <Box
                            w={'100%'}>
                            <LoaderItem />
                        </Box>
                    ) : (
                        <Box position={'relative'} w={'auto'} pb={'40px'}>
                            <SlideFade in={animateBox.isOpen} offsetY='20px'>
                                <Box
                                    boxShadow={'lg'}
                                    position={'relative'}
                                    p={'20px'}
                                    backgroundColor={'white'}>
                                    <Box>
                                        <Box
                                            color={'#171717'}
                                            px={'10px'}
                                            my={'5px'}
                                            fontSize={'16px'}
                                            fontWeight={700}>
                                            Select Component
                                        </Box>
                                        <Center flexWrap={'wrap'} justifyContent={'space-between'}>
                                            <Box w={'100%'} my={'5px'}>
                                                {
                                                    initialData && initialData.length > 0 ? (
                                                        <Box>
                                                            <FormLabel ml={'10px'} mb={'10px'} >City</FormLabel>
                                                            <Popover
                                                                isOpen={filterPopover.isOpen}
                                                                onOpen={filterPopover.onOpen}
                                                                onClose={() => {
                                                                    filterPopover.onClose();
                                                                    setSearchData('');
                                                                }}
                                                                placement='bottom'>
                                                                <PopoverTrigger>
                                                                    <Stack spacing={3} px={'10px'}>
                                                                        <InputGroup>
                                                                            <InputRightElement
                                                                                pointerEvents="none"
                                                                                // children={<BiMapPin color="rgba(162, 162, 162, 1)" />}
                                                                            />
                                                                            <Input
                                                                                color={'gray.500'}
                                                                                as={'button'}
                                                                                fontSize={'14px'}
                                                                                textAlign={'left'}
                                                                                border="1px"
                                                                                borderColor="#E9E9E9"
                                                                                _focus={{ outline: "none" }}
                                                                            >{selectedItems == '' ? "Select" : selectedItems}</Input>
                                                                        </InputGroup>
                                                                    </Stack>
                                                                </PopoverTrigger>
                                                                <PopoverContent
                                                                    w={'100%'}
                                                                    minW={'454px'}
                                                                    boxShadow="base"
                                                                    maxH="420px"
                                                                    overflowY="hidden"
                                                                    _focus={{ outline: "none" }}
                                                                >
                                                                    <PopoverArrow />
                                                                    <PopoverBody px="0">
                                                                        <Center justifyContent={'space-between'}>
                                                                            <Box w={'100%'}>
                                                                                <Stack spacing={3} px="10px">
                                                                                    <InputGroup>
                                                                                        <InputRightElement
                                                                                            pointerEvents="none"
                                                                                            children={
                                                                                                loaderSearch ?
                                                                                                    <CircularProgress size='20px' mx='auto' isIndeterminate color='#c6b378' />
                                                                                                    :
                                                                                                    // <FiSearch color="rgba(162, 162, 162, 1)" />
                                                                                                    <></>
                                                                                                }
                                                                                        />
                                                                                        <Input
                                                                                            _focus={{ outline: "none" }}
                                                                                            placeholder="Search by City"
                                                                                            fontSize="12px"
                                                                                            onChange={(e: any) => {
                                                                                                setSearchData(e.target.value);
                                                                                            }}
                                                                                        />
                                                                                    </InputGroup>
                                                                                </Stack>

                                                                                {
                                                                                    filteredData && filteredData.length > 0 && (
                                                                                        <VStack
                                                                                            w={'auto'}
                                                                                            divider={<StackDivider borderColor='gray.200' />}
                                                                                            spacing={4}
                                                                                            align='stretch'
                                                                                            mt={0}
                                                                                            className="filter-checkbox-wrapper"
                                                                                            overflowY="scroll"
                                                                                            maxH="300px"
                                                                                        >
                                                                                            <Box>
                                                                                                {
                                                                                                    filteredData.map((el: any, index: number) => {
                                                                                                        return (
                                                                                                            <Box
                                                                                                                p={3} key={index}
                                                                                                                onClick={() => handleSelectedItem(el)}
                                                                                                                cursor={'pointer'}
                                                                                                                _hover={{
                                                                                                                    backgroundColor: "rgba(249, 247, 242, 1)",
                                                                                                                }}>
                                                                                                                <Flex flexDir="row" justifyContent={'space-between'}>
                                                                                                                    <Box>
                                                                                                                        <Flex alignItems={'center'} flexDir="row">
                                                                                                                            <Text mx={'5px'} fontSize={'12px'} textTransform={'capitalize'} >{el[`${items}`]?.toLowerCase()}</Text>
                                                                                                                        </Flex>
                                                                                                                    </Box>
                                                                                                                </Flex>
                                                                                                            </Box>
                                                                                                        )
                                                                                                    })
                                                                                                }
                                                                                            </Box>
                                                                                        </VStack>
                                                                                    )
                                                                                }
                                                                            </Box>
                                                                        </Center>
                                                                    </PopoverBody>
                                                                </PopoverContent>
                                                            </Popover>
                                                        </Box>
                                                    ) : (
                                                        <Box>
                                                            <Stack spacing={3} px={'10px'}>
                                                            <FormLabel mb={0} >City</FormLabel>
                                                            <InputGroup>
                                                                <InputRightElement
                                                                    pointerEvents="none"
                                                                    children={
                                                                    // <FaCity color="rgba(162, 162, 162, 1)" />
                                                                    <></>
                                                                }
                                                                />
                                                                <Input
                                                                    _placeholder={{ opacity: 1, color: 'gray.500' }}
                                                                    _focus={{ outline: "none" }}
                                                                    placeholder="Fill City"
                                                                    fontSize="12px"
                                                                />
                                                            </InputGroup>
                                                        </Stack>
                                                        </Box>
                                                    )
                                                }
                                            </Box>
                                        </Center>
                                    </Box>
                                    {
                                        loaderBox && (
                                            <Box 
                                                left={'0'}
                                                top={'0'}
                                                position={'absolute'}
                                                w={'100%'}
                                                h={'100%'}
                                                backgroundColor={'rgba(255,255,255,.7)'}>
                                                <LoaderBox />
                                            </Box>
                                        )
                                    }
                                </Box>
                            </SlideFade>

                
                            


            
                        </Box>
                    )
                }
            </Center>
        );
    },
});
